<?php
date_default_timezone_set('Europe/Moscow');

error_reporting(E_ALL);
ini_set('display_errors', 0);
ini_set('log_errors', 1);
ini_set('error_log', 'errors_vote.log');

function customErrorHandler($errno, $errstr, $errfile, $errline)
{
    $randomNumber = rand(1000, 9999);
    $filename     = "error_{$randomNumber}.log";
    $dateTime     = date('Y-m-d H:i:s');
    $logMessage   = "[$dateTime] Error: $errstr in $errfile on line $errline" . PHP_EOL;
    $logMessage   .= "Stack trace:" . PHP_EOL;
    $backtrace    = debug_backtrace();
    foreach ($backtrace as $i => $trace) {
        if (isset($trace['file']) && isset($trace['line'])) {
            $logMessage .= "#$i {$trace['file']}({$trace['line']}): ";
            if (isset($trace['class'])) {
                $logMessage .= "{$trace['class']}{$trace['type']}";
            }
            $logMessage .= "{$trace['function']}(";
            if (isset($trace['args']) && count($trace['args']) > 0) {
                $args       = array_map('gettype', $trace['args']);
                $logMessage .= implode(', ', $args);
            }
            $logMessage .= ")" . PHP_EOL;
        }
    }
    file_put_contents($filename, $logMessage, FILE_APPEND);
}

set_error_handler("customErrorHandler");

$apiUrlTemplate = 'https://api.l2vote.bot/api/votes/server/%s/char-names';
$apiKey         = ''; // Вставьте сюда ключ вашего проекта

class Config
{

    public static array $servers = [
        // Если вам нужно добавить второй сервер, просто продублируйте этот блок для второго сервера
        // Начало блока данных от базы для первого сервера

      1 => [                  //Указать ид сервера (который получили в боте после добавления сервера)
        'db'     => [
          'host'    => '',
          'dbname'  => '',
          'user'    => '',
          'pass'    => '',
          'charset' => 'utf8mb4',
        ],
        'l2j'    => 'lucera3', // lucera3 или pw
        'domain' => 'Domain.net',
        'bonus'  => [
          ['item' => 57, 'count' => 1], //Выдать первый предмет
          ['item' => 57, 'count' => 2], //Выдать второй предмет (если не нужно, просто удалить или закоментировать строку)
          ['item' => 57, 'count' => 3], //Выдать третий предмет (если не нужно, просто удалить или закоментировать строку)	
        ],
      ],

        // Конец блока данных от базы для первого сервера	

    ];

}

class DatabaseConnection
{

    private $pdo;

    public function __construct($config)
    {
        $dsn     = "mysql:host={$config['host']};dbname={$config['dbname']};charset={$config['charset']}";
        $options = [
          PDO::ATTR_ERRMODE            => PDO::ERRMODE_EXCEPTION,
          PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
          PDO::ATTR_EMULATE_PREPARES   => false,
        ];

        try {
            $this->pdo = new PDO($dsn, $config['user'], $config['pass'], $options);
        } catch (\PDOException $e) {
            error_log("Ошибка подключения к базе данных {$config['dbname']}: " . $e->getMessage(), 3, 'errors_vote.log');
            throw new \PDOException($e->getMessage(), (int)$e->getCode());
        }
    }

    public function getConnection()
    {
        return $this->pdo;
    }

}

class ApiClient
{

    private string $apiUrlTemplate;

    private string $apiKey;

    public function __construct($apiUrlTemplate, $apiKey)
    {
        $this->apiUrlTemplate = $apiUrlTemplate;
        $this->apiKey         = $apiKey;
    }

    public function fetchVotes($serverId)
    {
        $apiUrl = sprintf($this->apiUrlTemplate, $serverId);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
          'X-API-KEY: ' . $this->apiKey,
          'Content-Type: application/json',
        ]);

        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if (curl_errno($ch)) {
            error_log('Ошибка cURL: ' . curl_error($ch), 3, 'errors_vote.log');
            throw new Exception('Ошибка cURL: ' . curl_error($ch));
        }

        curl_close($ch);

        if ($httpCode != 200) {
            error_log("Ошибка при запросе к API: HTTP $httpCode", 3, 'errors_vote.log');
            throw new Exception("Ошибка при запросе к API: HTTP $httpCode");
        }

        return json_decode($response, true);
    }

}

class VoteManager
{

    private ApiClient $apiClient;

    private string $today;

    private array $uniqueElements = [];

    private int $serverId;

    public function __construct($apiClient, $serverId)
    {
        $this->serverId  = $serverId;
        $this->apiClient = $apiClient;
        $this->today     = date('Y-m-d');
    }

    public function processVotes($serverConfig)
    {
        $votesListAPI = $this->apiClient->fetchVotes($this->serverId);
        $fileName     = "log/{$this->today}_votes_{$this->serverId}.json";

        if ( ! file_exists($fileName)) {
            file_put_contents($fileName, json_encode([]));
        }

        $mergedArray     = array_merge($this->getVotesList(), $votesListAPI);
        $lowercasedArray = array_map('strtolower', $mergedArray);
        $countValues     = array_count_values($lowercasedArray);

        $uniqueElements = [];
        foreach ($countValues as $value => $count) {
            if ($count === 1) {
                // Получаем оригинальные значения, которые имеют уникальные нижне-регистрированные представления
                foreach ($mergedArray as $originalValue) {
                    if (strtolower($originalValue) === $value) {
                        $uniqueElements[] = $originalValue;
                        break;
                    }
                }
            }
        }
        $this->uniqueElements = $uniqueElements;

        file_put_contents($fileName, json_encode($votesListAPI, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    private function getVotesList()
    {
        $fileName = "log/{$this->today}_votes_{$this->serverId}.json";
        if ( ! file_exists($fileName)) {
            return [];
        }

        return json_decode(file_get_contents($fileName), true);
    }

    public function sendBonus($config)
    {
        try {
            $db = new DatabaseConnection($config['db']);
        } catch (PDOException $e) {
            echo 'Ошибка подключения к базе данных: ' . $e->getMessage();
            exit;
        }
        $pdo = $db->getConnection();

        $users = $this->uniqueElements;
        if (count($users) == 0) {
            return null;
        }
        echo "\n======== Выдача бонусов для сервера : {$config['domain']} ($this->serverId) =========\n";
        $i = 0;
        foreach ($users as $user) {
            $i++;
            $selectCharSQL = "SELECT `obj_Id` FROM `characters` WHERE char_name = :charName";
            $selectChar    = $pdo->prepare($selectCharSQL);
            $selectChar->execute([
              'charName' => $user,
            ]);
            if ($selectChar->rowCount() == 0) {
                // Пользователь не найден
                error_log("Пользователь {$user} не найден", 3, 'errors_vote.log');
                continue;
            }
            $charId = $selectChar->fetch()['obj_Id'];

            $bonus = $config['bonus'];
            $l2j   = $config['l2j'];
            foreach ($bonus as $bonusItem) {
                $itemId                = $bonusItem['item'];
                $count                 = $bonusItem['count'];
                $insertItemsDelayedSQL = '';

                switch ($l2j) {
                    case 'lucera3':
                        $insertItemsDelayedSQL = "INSERT INTO `items_delayed` (owner_id, item_id, count, enchant_level, flags, payment_status, description) VALUES (:ownerId, :itemId, :count, 0, 0, 0, 'L2Vote_bot')";
                        break;
                    case 'pw':
                        $insertItemsDelayedSQL = "INSERT INTO z_queued_items (char_id, name, item_id, item_count, item_ench, aug_id, aug_lvl, status) VALUES (:ownerId, 'L2Vote_bot', :itemId, :count, 0, 0, 0, 0)";
                        break;
                }

                $insertItemsDelayed = $pdo->prepare($insertItemsDelayedSQL);
                $insertItemsDelayed->execute([
                  'ownerId' => $charId,
                  'itemId'  => $itemId,
                  'count'   => $count,
                ]);
                $timedate = date('H:i:s');
                $message  = sprintf("[%s] Выдано игроку %s бонус %s (%s)\n", $timedate, $user, $itemId, $count);
                file_put_contents("log/{$this->today}_log_{$this->serverId}.json", $message, FILE_APPEND);
                echo $message;
            }
        }
    }

}

// Проверка существования папки log и создаем её если нету.
if ( ! file_exists('log')) {
    mkdir('log', 0777, true);
}

foreach (Config::$servers as $serverId => $serverConfig) {
    $apiClient   = new ApiClient($apiUrlTemplate, $apiKey);
    $voteManager = new VoteManager($apiClient, $serverId);
    $voteManager->processVotes($serverConfig);
    $voteManager->sendBonus($serverConfig);
}
